@@ -1,5 +1,5 @@
 /*
- * "$Id: language.c 5753 2006-07-18 19:53:24Z mike $"
+ * "$Id: language.c 5980 2006-09-21 19:01:55Z mike $"
  *
  *   I18N/language support for the Common UNIX Printing System (CUPS).
  *
@@ -406,14 +406,6 @@ cupsLangGet(const char *language)	/* I - Language or locale */
 
         *ptr = '\0';
       }
-      else
-      {
-       /*
-        * Default to UTF-8...
-	*/
-
-        strcpy(charset, "UTF8");
-      }
 
      /*
       * Get the locale for messages from the LC_MESSAGES locale setting...
@@ -473,6 +465,13 @@ cupsLangGet(const char *language)	/* I - Language or locale */
   }
 #endif /* CODESET */
 
+ /*
+  * If we don't have a character set by now, default to UTF-8...
+  */
+
+  if (!charset[0])
+    strcpy(charset, "UTF8");
+
  /*
   * Parse the language string passed in to a locale string. "C" is the
   * standard POSIX locale and is copied unchanged.  Otherwise the
@@ -1288,6 +1287,8 @@ cups_unquote(char       *d,		/* O - Unquoted string */
 	  *d = *d * 8 + *s - '0';
 	  s ++;
 	}
+
+	d ++;
       }
       else
       {
@@ -1312,5 +1313,5 @@ cups_unquote(char       *d,		/* O - Unquoted string */
 
 
 /*
- * End of "$Id: language.c 5753 2006-07-18 19:53:24Z mike $".
+ * End of "$Id: language.c 5980 2006-09-21 19:01:55Z mike $".
  */
