@@ -1,9 +1,9 @@
 /*
- * "$Id: mark.c,v 1.25 2002/01/02 17:58:40 mike Exp $"
+ * "$Id: mark.c 5700 2006-06-26 19:20:39Z mike $"
  *
  *   Option marking routines for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2002 by Easy Software Products, all rights reserved.
+ *   Copyright 1997-2006 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -15,20 +15,24 @@
  *       Attn: CUPS Licensing Information
  *       Easy Software Products
  *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3111 USA
+ *       Hollywood, Maryland 20636 USA
  *
- *       Voice: (301) 373-9603
+ *       Voice: (301) 373-9600
  *       EMail: cups-info@cups.org
  *         WWW: http://www.cups.org
  *
  *   PostScript is a trademark of Adobe Systems, Inc.
  *
+ *   This file is subject to the Apple OS-Developed Software exception.
+ *
  * Contents:
  *
  *   ppdConflicts()        - Check to see if there are any conflicts.
  *   ppdFindChoice()       - Return a pointer to an option choice.
  *   ppdFindMarkedChoice() - Return the marked choice for the specified option.
  *   ppdFindOption()       - Return a pointer to the specified option.
+ *   ppdFirstOption()      - Return the first option in the PPD file.
+ *   ppdNextOption()       - Return the next option in the PPD file.
  *   ppdIsMarked()         - Check to see if an option is marked...
  *   ppdMarkDefaults()     - Mark all default options in the PPD file.
  *   ppdMarkOption()       - Mark an option in a PPD file.
@@ -39,7 +43,7 @@
  * Include necessary headers...
  */
 
-#include "ppd.h"
+#include "cups.h"
 #include "string.h"
 #include "debug.h"
 
@@ -237,28 +241,46 @@ ppd_option_t *				/* O - Pointer to option or NULL */
 ppdFindOption(ppd_file_t *ppd,		/* I - PPD file data */
               const char *option)	/* I - Option/Keyword name */
 {
-  int		i, j, k;	/* Looping vars */
-  ppd_option_t	*o;		/* Pointer to option */
-  ppd_group_t	*g,		/* Pointer to group */
-		*sg;		/* Pointer to subgroup */
+ /*
+  * Range check input...
+  */
 
-
-  if (ppd == NULL || option == NULL)
+  if (!ppd || !option)
     return (NULL);
 
-  for (i = ppd->num_groups, g = ppd->groups; i > 0; i --, g ++)
+  if (ppd->options)
   {
-    for (j = g->num_options, o = g->options; j > 0; j --, o ++)
-      if (strcasecmp(o->keyword, option) == 0)
-	return (o);
+   /*
+    * Search in the array...
+    */
 
-    for (j = g->num_subgroups, sg = g->subgroups; j > 0; j --, sg ++)
-      for (k = sg->num_options, o = sg->options; k > 0; k --, o ++)
-	if (strcasecmp(o->keyword, option) == 0)
-	  return (o);
+    ppd_option_t	key;		/* Option search key */
+
+
+    strlcpy(key.keyword, option, sizeof(key.keyword));
+
+    return ((ppd_option_t *)cupsArrayFind(ppd->options, &key));
   }
+  else
+  {
+   /*
+    * Search in each group...
+    */
 
-  return (NULL);
+    int			i, j;		/* Looping vars */
+    ppd_group_t		*group;		/* Current group */
+    ppd_option_t	*optptr;	/* Current option */
+
+
+    for (i = ppd->num_groups, group = ppd->groups; i > 0; i --, group ++)
+      for (j = group->num_options, optptr = group->options;
+           j > 0;
+	   j --, optptr ++)
+        if (!strcasecmp(optptr->keyword, option))
+	  return (optptr);
+
+    return (NULL);
+  }
 }
 
 
@@ -321,96 +343,323 @@ ppdMarkOption(ppd_file_t *ppd,		/* I - PPD file record */
               const char *option,	/* I - Keyword */
               const char *choice)	/* I - Option name */
 {
-  int		i;		/* Looping var */
-  ppd_option_t	*o;		/* Option pointer */
-  ppd_choice_t	*c;		/* Choice pointer */
+  int		i, j;			/* Looping vars */
+  ppd_option_t	*o;			/* Option pointer */
+  ppd_choice_t	*c;			/* Choice pointer */
+  struct lconv	*loc;			/* Locale data */
 
 
-  if (ppd == NULL)
+  DEBUG_printf(("ppdMarkOption(ppd=%p, option=\"%s\", choice=\"%s\")\n",
+        	ppd, option, choice));
+
+ /*
+  * Range check input...
+  */
+
+  if (!ppd || !option || !choice)
     return (0);
 
-  if (strcasecmp(option, "PageSize") == 0 && strncasecmp(choice, "Custom.", 7) == 0)
-  {
-   /*
-    * Handle variable page sizes...
-    */
+ /*
+  * AP_D_InputSlot is the "default input slot" on MacOS X, and setting
+  * it clears the regular InputSlot choices...
+  */
 
-    ppdPageSize(ppd, choice);
-    choice = "Custom";
+  if (!strcasecmp(option, "AP_D_InputSlot"))
+  {
+    if ((o = ppdFindOption(ppd, "InputSlot")) != NULL)
+      for (i = 0; i < o->num_choices; i ++)
+	o->choices[i].marked = 0;
   }
 
+ /*
+  * Check for custom options...
+  */
+
   if ((o = ppdFindOption(ppd, option)) == NULL)
     return (0);
 
-  for (i = o->num_choices, c = o->choices; i > 0; i --, c ++)
-    if (strcasecmp(c->choice, choice) == 0)
-      break;
+  loc = localeconv();
 
-  if (i)
+  if (!strncasecmp(choice, "Custom.", 7))
   {
    /*
-    * Option found; mark it and then handle unmarking any other options.
+    * Handle a custom option...
     */
 
-    c->marked = 1;
+    if ((c = ppdFindChoice(o, "Custom")) == NULL)
+      return (0);
 
-    if (o->ui != PPD_UI_PICKMANY)
-      for (i = o->num_choices, c = o->choices; i > 0; i --, c ++)
-	if (strcasecmp(c->choice, choice) != 0)
-          c->marked = 0;
-
-    if (strcasecmp(option, "PageSize") == 0 || strcasecmp(option, "PageRegion") == 0)
+    if (!strcasecmp(option, "PageSize"))
     {
      /*
-      * Mark current page size...
+      * Handle custom page sizes...
       */
 
-      for (i = 0; i < ppd->num_sizes; i ++)
-	ppd->sizes[i].marked = strcasecmp(ppd->sizes[i].name, choice) == 0;
-
+      ppdPageSize(ppd, choice);
+    }
+    else
+    {
      /*
-      * Unmark the current PageSize or PageRegion setting, as appropriate...
+      * Handle other custom options...
       */
 
-      if (strcasecmp(option, "PageSize") == 0)
+      ppd_coption_t	*coption;	/* Custom option */
+      ppd_cparam_t	*cparam;	/* Custom parameter */
+      char		*units;		/* Custom points units */
+
+
+      if ((coption = ppdFindCustomOption(ppd, option)) != NULL)
       {
-	if ((o = ppdFindOption(ppd, "PageRegion")) != NULL)
-	  for (i = 0; i < o->num_choices; i ++)
-            o->choices[i].marked = 0;
-      }
-      else
-      {
-	if ((o = ppdFindOption(ppd, "PageSize")) != NULL)
-	  for (i = 0; i < o->num_choices; i ++)
-            o->choices[i].marked = 0;
+        if ((cparam = (ppd_cparam_t *)cupsArrayFirst(coption->params)) == NULL)
+	  return (0);
+
+        switch (cparam->type)
+	{
+	  case PPD_CUSTOM_CURVE :
+	  case PPD_CUSTOM_INVCURVE :
+	  case PPD_CUSTOM_REAL :
+	      cparam->current.custom_real = _cupsStrScand(choice + 7, NULL,
+	                                                  loc);
+	      break;
+
+	  case PPD_CUSTOM_POINTS :
+	      cparam->current.custom_points = _cupsStrScand(choice + 7,
+	                                                    &units, loc);
+
+              if (units)
+	      {
+        	if (!strcasecmp(units, "cm"))
+	          cparam->current.custom_points *= 72.0 / 2.54;	      
+        	else if (!strcasecmp(units, "mm"))
+	          cparam->current.custom_points *= 72.0 / 25.4;	      
+        	else if (!strcasecmp(units, "m"))
+	          cparam->current.custom_points *= 72.0 / 0.0254;	      
+        	else if (!strcasecmp(units, "in"))
+	          cparam->current.custom_points *= 72.0;	      
+        	else if (!strcasecmp(units, "ft"))
+	          cparam->current.custom_points *= 12 * 72.0;	      
+              }
+	      break;
+
+	  case PPD_CUSTOM_INT :
+	      cparam->current.custom_int = atoi(choice + 7);
+	      break;
+
+	  case PPD_CUSTOM_PASSCODE :
+	  case PPD_CUSTOM_PASSWORD :
+	  case PPD_CUSTOM_STRING :
+	      if (cparam->current.custom_string)
+	        free(cparam->current.custom_string);
+
+	      cparam->current.custom_string = strdup(choice + 7);
+	      break;
+	}
       }
     }
-    else if (strcasecmp(option, "InputSlot") == 0)
-    {
-     /*
-      * Unmark ManualFeed option...
-      */
 
-      if ((o = ppdFindOption(ppd, "ManualFeed")) != NULL)
-	for (i = 0; i < o->num_choices; i ++)
-          o->choices[i].marked = 0;
-    }
-    else if (strcasecmp(option, "ManualFeed") == 0)
-    {
-     /*
-      * Unmark InputSlot option...
-      */
+   /*
+    * Make sure that we keep the option marked below...
+    */
 
-      if ((o = ppdFindOption(ppd, "InputSlot")) != NULL)
-	for (i = 0; i < o->num_choices; i ++)
-          o->choices[i].marked = 0;
+    choice = "Custom";
+  }
+  else if (choice[0] == '{')
+  {
+   /*
+    * Handle multi-value custom options...
+    */
+
+    ppd_coption_t	*coption;	/* Custom option */
+    ppd_cparam_t	*cparam;	/* Custom parameter */
+    char		*units;		/* Custom points units */
+    int			num_vals;	/* Number of values */
+    cups_option_t	*vals,		/* Values */
+			*val;		/* Value */
+
+
+    if ((c = ppdFindChoice(o, "Custom")) == NULL)
+      return (0);
+
+    if ((coption = ppdFindCustomOption(ppd, option)) != NULL)
+    {
+      num_vals = cupsParseOptions(choice + 1, 0, &vals);
+
+      for (i = 0, val = vals; i < num_vals; i ++, val ++)
+      {
+        if ((cparam = ppdFindCustomParam(coption, val->name)) == NULL)
+	  continue;
+
+	switch (cparam->type)
+	{
+	  case PPD_CUSTOM_CURVE :
+	  case PPD_CUSTOM_INVCURVE :
+	  case PPD_CUSTOM_REAL :
+	      cparam->current.custom_real = _cupsStrScand(val->value, NULL,
+	                                                  loc);
+	      break;
+
+	  case PPD_CUSTOM_POINTS :
+	      cparam->current.custom_points = _cupsStrScand(val->value, &units,
+	                                                    loc);
+
+	      if (units)
+	      {
+        	if (!strcasecmp(units, "cm"))
+		  cparam->current.custom_points *= 72.0 / 2.54;	      
+        	else if (!strcasecmp(units, "mm"))
+		  cparam->current.custom_points *= 72.0 / 25.4;	      
+        	else if (!strcasecmp(units, "m"))
+		  cparam->current.custom_points *= 72.0 / 0.0254;	      
+        	else if (!strcasecmp(units, "in"))
+		  cparam->current.custom_points *= 72.0;	      
+        	else if (!strcasecmp(units, "ft"))
+		  cparam->current.custom_points *= 12 * 72.0;	      
+	      }
+	      break;
+
+	  case PPD_CUSTOM_INT :
+	      cparam->current.custom_int = atoi(val->value);
+	      break;
+
+	  case PPD_CUSTOM_PASSCODE :
+	  case PPD_CUSTOM_PASSWORD :
+	  case PPD_CUSTOM_STRING :
+	      if (cparam->current.custom_string)
+		free(cparam->current.custom_string);
+
+	      cparam->current.custom_string = strdup(val->value);
+	      break;
+	}
+      }
+
+      cupsFreeOptions(num_vals, vals);
     }
   }
+  else
+  {
+    for (i = o->num_choices, c = o->choices; i > 0; i --, c ++)
+      if (!strcasecmp(c->choice, choice))
+        break;
+
+    if (!i)
+      return (0);
+  }
+
+ /*
+  * Option found; mark it and then handle unmarking any other options.
+  */
+
+  c->marked = 1;
+
+  if (o->ui != PPD_UI_PICKMANY)
+  {
+   /*
+    * Unmark all other choices...
+    */
+
+    for (i = o->num_choices, c = o->choices; i > 0; i --, c ++)
+      if (strcasecmp(c->choice, choice))
+      {
+        c->marked = 0;
+
+	if (!strcasecmp(option, "PageSize") ||
+	    !strcasecmp(option, "PageRegion"))
+	{
+	 /*
+	  * Mark current page size...
+	  */
+
+	  for (j = 0; j < ppd->num_sizes; j ++)
+	    ppd->sizes[j].marked = !strcasecmp(ppd->sizes[j].name,
+		                               choice);
+
+	 /*
+	  * Unmark the current PageSize or PageRegion setting, as
+	  * appropriate...
+	  */
+
+	  if (!strcasecmp(option, "PageSize"))
+	  {
+	    if ((o = ppdFindOption(ppd, "PageRegion")) != NULL)
+	      for (j = 0; j < o->num_choices; j ++)
+        	o->choices[j].marked = 0;
+	  }
+	  else
+	  {
+	    if ((o = ppdFindOption(ppd, "PageSize")) != NULL)
+	      for (j = 0; j < o->num_choices; j ++)
+        	o->choices[j].marked = 0;
+	  }
+	}
+	else if (!strcasecmp(option, "InputSlot"))
+	{
+	 /*
+	  * Unmark ManualFeed True and possibly mark ManualFeed False
+	  * option...
+	  */
+
+	  if ((o = ppdFindOption(ppd, "ManualFeed")) != NULL)
+	    for (j = 0; j < o->num_choices; j ++)
+              o->choices[j].marked = !strcasecmp(o->choices[j].choice, "False");
+	}
+	else if (!strcasecmp(option, "ManualFeed") &&
+	         !strcasecmp(choice, "True"))
+	{
+	 /*
+	  * Unmark InputSlot option...
+	  */
+
+	  if ((o = ppdFindOption(ppd, "InputSlot")) != NULL)
+	    for (j = 0; j < o->num_choices; j ++)
+              o->choices[j].marked = 0;
+	}
+      }
+  }
+
+ /*
+  * Return the number of conflicts...
+  */
 
   return (ppdConflicts(ppd));
 }
 
 
+/*
+ * 'ppdFirstOption()' - Return the first option in the PPD file.
+ *
+ * Options are returned from all groups in sorted order.
+ *
+ * @since CUPS 1.2@
+ */
+
+ppd_option_t *				/* O - First option or NULL */
+ppdFirstOption(ppd_file_t *ppd)		/* I - PPD file */
+{
+  if (!ppd)
+    return (NULL);
+  else
+    return ((ppd_option_t *)cupsArrayFirst(ppd->options));
+}
+
+
+/*
+ * 'ppdNextOption()' - Return the next option in the PPD file.
+ *
+ * Options are returned from all groups in sorted order.
+ *
+ * @since CUPS 1.2@
+ */
+
+ppd_option_t *				/* O - Next option or NULL */
+ppdNextOption(ppd_file_t *ppd)		/* I - PPD file */
+{
+  if (!ppd)
+    return (NULL);
+  else
+    return ((ppd_option_t *)cupsArrayNext(ppd->options));
+}
+
+
 /*
  * 'ppd_defaults()' - Set the defaults for this group and all sub-groups.
  */
@@ -437,5 +686,5 @@ ppd_defaults(ppd_file_t  *ppd,	/* I - PPD file */
 
 
 /*
- * End of "$Id: mark.c,v 1.25 2002/01/02 17:58:40 mike Exp $".
+ * End of "$Id: mark.c 5700 2006-06-26 19:20:39Z mike $".
  */
